""" Copyright start
  Copyright (C) 2008 - 2025 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import json
import logging

from rest_framework import status

from xfutils.constants import AUTH_CLAIM_TENANT_ID, AUTH_CLAIM_TENANT_UUID, AUTH_CLAIM_ACCESS_TOKEN, API_KEY_EXTRACTION_FAILED, \
    API_KEY
from xfutils.exceptions import XFException
from xfutils.import_export.error_message import xf_auth_40
from xfutils.utilities.security_utils import decrypt_data, get_system_level_secret

logger = logging.getLogger(__name__)


class ApiKeyHelper:
    @staticmethod
    def extract_information_from_key(api_key):
        """
        Get tenant id from api key
        Args:
            api_key: api key
        Returns:
            tenant_uuid: tenant id extracted from api key
            api_key : api key extracted from api key
        """
        try:
            secret = get_system_level_secret()
            _key = decrypt_data(secret, api_key)
            api_key = json.loads(_key)
            decrypted_key = api_key[AUTH_CLAIM_ACCESS_TOKEN]
            tenant_len = 36
            # tenant_len = int(get_tenant_id_size_api_key())
            claim = {
                AUTH_CLAIM_TENANT_ID : api_key[AUTH_CLAIM_TENANT_ID],
                AUTH_CLAIM_TENANT_UUID : decrypted_key[:tenant_len],
                API_KEY : decrypted_key[tenant_len:]
            }
            return claim
        except Exception as e:
            logger.error(API_KEY_EXTRACTION_FAILED.format(api_key, e))
            raise XFException(status.HTTP_401_UNAUTHORIZED, xf_auth_40)

