#!/bin/bash
# Copyright start
# Copyright (C) 2008 - 2025 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end

agent_version="@agentVersion@"
time_stamp=$(date +"%F"-"%s")

get_password() {

    if [ "$is_upgrade" = "true" ]; then
      echo "Upgrade is enabled."
      pgpw=$(grep 'db_password:' /opt/integration/configs/agent/db_config.yml | awk '{print $2}')
      if [ -n "$pgpw" ]; then
        return
      fi
    fi

    local pgpw_1
    local pgpw_2

    read -s -p "Enter a password to be used for the Agent database: " pgpw_1
    printf "\n"
    if [[ -z $pgpw_1 ]]; then
        printf "Password string may not be empty. Please try again.\n"
        return
    fi
    if [[ ! $pgpw_1 =~ .*[a-z].* ]] || [[ ! $pgpw_1 =~ .*[A-Z].* ]] || [[ ! $pgpw_1 =~ .*[0-9].* ]] || [[ ! ${#pgpw_1} -ge 10 ]] || [[ ! ${#pgpw_1} -le 64 ]]; then
        printf "Passwords must contain one lowercase letter, one uppercase letter, and one numerical digit, and must be between 10 and 64 characters long. Please ensure your password meets these requirements and try again.\n"
        return
    fi
    read -s -p "Re-enter chosen password: " pgpw_2
    printf "\n"
    if [[ $pgpw_1 != $pgpw_2 ]]; then
        printf "Password strings must match. Please try again.\n"
        return
    else
        pgpw="$pgpw_1"
        return
    fi
}

check_if_upgrade() {
    local package_name="xf-integration-agent"
    local config_file="/opt/integration/configs/agent/db_config.yml"
    if rpm -qa | grep -q "$package_name"; then
        if [ -f "$config_file" ]; then
            is_upgrade="true"
        fi
    fi
}

script_setup() {
  agent_user="xfabric"
  agent_group="xfabric"
  prefix="/opt/integration/"
  pg_version="16"
  pg_hba_file="/var/lib/pgsql/$pg_version/data/pg_hba.conf"

  build_number="@agentBuildNumber@"
  agent_id="@agentId@"
  agent_rpm_name="xfagent.x86_64.rpm"
  product_yum_server="repo.fortisoar.fortinet.com"
  tenant_id="@tenantid@"
  schema="@schema@"
  rmq_hostname="@rmqPublicHost@"
  agent_config="# Copyright start
# Copyright (C) 2008 - 2025 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end
rmq_public:
  host: @rmqPublicHost@
  sni: @rmqPublicSni@
  port: 5671
  vhost: @vhost@
  user: @user@
  ssl: true
  heartbeat: 120
  caCertPath: /opt/xf/configs/ca_cert.pem
  caChainBundle: /opt/xf/configs/ca_chain_bundle.pem
  clientCertFileName: /opt/xf/configs/client_cert.pem
  clientKeyFileName: /opt/xf/configs/client_key.pem
xf.instance.lwAgent: true
xf.instance.agentId: @agentId@
xf.instance.applianceUuid: @applianceUuid@
xf.instance.applianceKey: |
  @applianceKey@
database:
  db_external: false
  db_host: 127.0.0.1
  db_port: 5432
  db_ssl_cert_path: ''
  db_use_ssl: false
  db_user: $pguser
  db_name: integration"
  ini_config="; Copyright start
; Copyright (C) 2008 - 2025 Fortinet Inc.
; All rights reserved.
; FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
; Copyright end

[application]
debug = false
app_host = @hostname@
THREAD_POOL_WORKER = 16
FSR_NULL = 'FSR_NULL'
conn_rpm_temp_dir = /opt/cyops-integrations/cyops_connector_rpm
auth_host = @hostname@
wf_host = @hostname@
integration_host = @hostname@

[logging]
app_log_file_path = /var/log/cyops/cyops-integrations/connectors.log
app_log_file_size = 104857600
app_log_file_backup_count = 10
root_logger_level = WARNING
connector_logger_level = INFO
django_logger_level = INFO
django_backend_logger_level = WARNING

#########
# Container config starts here
#########
[container]
# In K8s
# You can use the suffix m to mean milli.
# For example 100m CPU, 100 milliCPU, and 0.1 CPU are all the same.
# Precision finer than 1m is not allowed.
# For simplicity, do not use 'm' and instead specify in just number.
#
cpu = 0.2

#########
# Gunicorn config starts here
#########
[gunicorn]
# Can be debug, info, warning, error, critical
loglevel = debug
port = 9595
# In sec
graceful_timeout = 900
timeout = 600
max_requests = 2000
max_requests_jitter = 100"
  connectors_json='@connectorsJson@'
  ca_cert="@caCert@"
  ca_chain_bundle="@caChainBundle@"
  client_cert="@clientCert@"
  client_key="@clientKey@"
  pwd=$(pwd)
  f_script_name="xfagent-$agent_version-$build_number.x86_64.rpm"

  pg_already_installed=0
  if rpm -q postgresql$pg_version-server; then
    pg_already_installed=1
  fi

  xf_install_needed=1
  if rpm -q xf-integration-agent; then
    myver="$(rpm -q --qf '%{VERSION}' xf-integration-agent)"
    if [ "$myver" == "$agent_version"]; then
      xf_install_needed=0
    fi
  fi

}

set_locale() {
  export LANG=en_US.UTF-8
  export LANGUAGE=en_US.UTF-8
  export LC_COLLATE=C
  export LC_CTYPE=en_US.UTF-8
}

install_agent_rpm() {
    # Download rpm file and install
    # Exit if the rpm file does not exist
    if [ "$xf_install_needed" == "1" ]; then
      echo "======================================================================================================"
      echo "Installing Agent Package"
      echo "======================================================================================================"
      yum list xf-integration-agent-$agent_version-$build_number > /dev/null
      if [ $? -eq 0 ]
      then
          echo "Installing Agent package"
          yum -y --nogpgcheck install xf-integration-agent-$agent_version-$build_number
      else
          echo "Package xf-integration-agent-$agent_version-$build_number is not present in repository"
          exit 1
      fi
    fi
}

set_env_variables() {
    echo "======================================================================================================"
    echo "Setting Environment Variables"
    echo "======================================================================================================"
    touch /etc/environment

    echo export PYTHONUNBUFFERED=1 >> /etc/environment
    echo export PYTHONDONTWRITEBYTECODE=1 >> /etc/environment
    # export PYTHONUNBUFFERED=1
    # export PYTHONDONTWRITEBYTECODE=1

    # Add environment variables for agent detection
    echo export ENV="agent" >> /etc/environment
    echo export TENANT_ID="$tenant_id" >> /etc/environment
    echo export SCHEMA="$schema" >> /etc/environment
    # export ENV="agent"
    # export TENANT_ID="$tenant_id"
    # export SCHEMA="$schema"

}

check_rmq_reachable(){
  if command -v ping &> /dev/null; then
    if ping -c 2 "$rmq_hostname" &> /dev/null; then
        echo "RabbitMQ Host $rmq_hostname is reachable"
    else
        echo "RabbitMQ Host $rmq_hostname is unreachable"
        exit 1
    fi
  else
    echo "PING command not found. Cannot check availability of RabbitMQ server."
  fi
}

add_extra_hosts() {
    local rmq_ip="69.167.115.175"
    local rmq_hostname="uat-agent-rmq.pool1.ca-west-1.xf.fortisoar.forticloud.com"
    printf "%s\t%s\n" "$rmq_ip" "$rmq_hostname" >> /etc/hosts
}

populate_config_files() {
    # Populate config files with the correct values
    echo "======================================================================================================"
    echo "Creating Configuration Files"
    echo "======================================================================================================"
    echo "$agent_config" > /opt/integration/configs/agent/agent_config.yml
    echo "$ini_config" > /opt/integration/configs/agent/config.ini
    touch /opt/xf/configs/ca_cert.pem
    echo "$ca_cert" > /opt/xf/configs/ca_cert.pem
    touch /opt/xf/configs/client_cert.pem
    echo "$client_cert" > /opt/xf/configs/client_cert.pem
    touch /opt/xf/configs/ca_chain_bundle.pem
    echo "$ca_chain_bundle" > /opt/xf/configs/ca_chain_bundle.pem
    touch /opt/xf/configs/client_key.pem
    echo "$client_key" > /opt/xf/configs/client_key.pem
    sed -i.bak "s#%tenant_id%#$tenant_id#g" /opt/integration/package.d/gunicorn.service
    sed -i.bak "s#%schema%#$schema#g" /opt/integration/package.d/gunicorn.service
    rm -f /opt/integration/package.d/gunicorn.service.bak
}

populate_connectors_file() {
  echo "======================================================================================================"
  echo "Creating Connectors File"
  echo "======================================================================================================"
  touch /opt/integration/connectors/agent_connectors.json
  echo "$connectors_json" > /opt/integration/connectors/agent_connectors.json
}

disable_existing_postgres_repo() {
  # For Fix of following error
  # All matches were filtered out by modular filtering for argument: postgresql14-server
  if [ "$pg_already_installed" -eq "0" ]; then
    sudo dnf -qy module disable postgresql
    yum clean all
  fi
}

add_xf_repo() {
  local xf_app_repo="[xf-app]
name=XFabric Application Repository
baseurl=https://$product_yum_server/xf/$agent_version/agent/x86_64/
gpgcheck=0
enabled=1
sslverify=false"
  if [ "$xf_install_needed" -eq "1" ]; then
    echo "$xf_app_repo" >/etc/yum.repos.d/xf-app.repo
    sudo chmod 644 /etc/yum.repos.d/xf-app.repo
  fi
}

add_postgres_repo() {
  local postgres_repo="[fsr-pgdg]
name=FortiSOAR PostgreSQL repository
baseurl=https://$product_yum_server/xf/$agent_version/third-party/postgres/$pg_version/rhel-\$releasever-\$basearch/
enabled=1
gpgcheck=1
gpgkey=https://$product_yum_server/xf/$agent_version/third-party/postgres/RPM-GPG-KEY-PGDG-$pg_version
sslverify=false"
  if [ "$pg_already_installed" -eq "0" ]; then
    echo "$postgres_repo" >/etc/yum.repos.d/fsr-pgdg.repo
    sudo chmod 644 /etc/yum.repos.d/fsr-pgdg.repo
  fi
}

install_configure_postgres() {
  local pg_hba_conf="/var/lib/pgsql/${pg_version}/data/pg_hba.conf"
  local is_initdb='false'

  if [ "$pg_already_installed" -eq "0" ]; then
    echo "======================================================================================================"
    echo "Installing Postgres Server"
    echo "======================================================================================================"
    yum install -y postgresql${pg_version}-server --nogpgcheck
    if [ $? -ne 0 ]; then
      echo "Unable to install PostgreSQL, Exiting the installer"
      echo "Validate connectivity with $product_yum_server or check host entry"
      exit 1
    fi
  fi

  # Initialize PostgreSQL only if the data directory is missing
  if [ ! -f "$pg_hba_conf" ]; then
    echo "======================================================================================================"
    echo "Initializing Postgres Server"
    echo "======================================================================================================"
    /usr/pgsql-${pg_version}/bin/postgresql-${pg_version}-setup initdb
    is_initdb='true'
  fi

  if [ "$is_upgrade" = "false" ]; then
    echo "Encrypting password" >&3
    encrypted_pass=$(sudo -u "$agent_user" "$prefix/.env/bin/python3" /opt/integration/manage.py manage_password encrypt "$pgpw")

    if [ -n "$encrypted_pass" ]; then
      printf "database:\n  db_password: %s\n" "$encrypted_pass" > /opt/integration/configs/agent/db_config.yml
      echo "Created db_config.yml" >&3
    fi
  fi

  if [ "$is_initdb" = "true" ]; then
    echo "Enabling and starting Postgres Server"
    systemctl enable postgresql-${pg_version}
    systemctl start postgresql-${pg_version}.service
  fi

  if [ "$is_upgrade" = "false" ]; then
    create_pg_user_and_db
    setup_pg_hba

    # Check if "phoenix" appears in pg_hba.conf
    if grep -iq "phoenix" "$pg_hba_conf"; then
        echo "Found 'phoenix' in pg_hba.conf. Reloading PostgreSQL configuration..."
        if systemctl is-active --quiet postgresql-${pg_version}; then
            psql -U postgres -c "SELECT pg_reload_conf()" postgres
        else
            echo "PostgreSQL is not running, skipping pg_reload_conf"
            exit 1
        fi
    else
        echo "'phoenix' not found. Restarting PostgreSQL service..."
        systemctl restart postgresql-${pg_version}.service
    fi
  fi
}

create_pg_user_and_db() {
  sudo -Hiu postgres psql -U postgres << XFABRICHEREDOCDELIM >&3 2>&3
SELECT pg_terminate_backend(pid) FROM pg_stat_activity WHERE datname = 'integration';

DROP DATABASE IF EXISTS integration;

DROP USER IF EXISTS "$pguser";
DO
\$XFabricPWDelim\$
BEGIN
    EXECUTE format('CREATE USER %I WITH SUPERUSER PASSWORD %L', '$pguser', '$pgpw');
END
\$XFabricPWDelim\$;

CREATE DATABASE integration OWNER "$pguser";
XFABRICHEREDOCDELIM
}

setup_pg_hba() {
    local pg_hba_conf="/var/lib/pgsql/${pg_version}/data/pg_hba.conf"
    local backup_dir="/var/lib/pg_hba_backup"
    local backup_file="$backup_dir/pg_hba.conf.$(date +%Y%m%d%H%M%S).bak"
    # Ensure backup directory exists
    mkdir -p "$backup_dir"
    # Backup the original file before modifying
    cp "$pg_hba_conf" "$backup_file"

    # Define the required lines
    local local_postgres="local   all             postgres                                trust"
    local local_cyberpgsql="local   integration             $pguser                              md5"
    local ipv4_cyberpgsql="host    integration             $pguser      127.0.0.1/32            md5"
    local ipv6_cyberpgsql="host    integration             $pguser             ::1/128          md5"

    # Check if all required entries already exist
    if grep -Fxq "$local_postgres" "$pg_hba_conf" && \
       grep -Fxq "$local_cyberpgsql" "$pg_hba_conf" && \
       grep -Fxq "$ipv4_cyberpgsql" "$pg_hba_conf" && \
       grep -Fxq "$ipv6_cyberpgsql" "$pg_hba_conf"; then
        echo "No changes needed. All required entries already exist." >&3
        return
    fi

    # Use awk to process the file while keeping all existing content intact
    awk -v l1="$local_postgres" \
        -v l2="$local_cyberpgsql" \
        -v h1="$ipv4_cyberpgsql" \
        -v h2="$ipv6_cyberpgsql" \
        -v added_local=0 -v added_ipv4=0 -v added_ipv6=0 '
    BEGIN { print "# Updated pg_hba.conf with new entries (if missing)" }
    /^# "local" is for Unix domain socket connections only/ {
        print
        if (!added_local) { print l1; print l2; added_local=1 }
        next
    }
    /^# IPv4 local connections:/ {
        print
        if (!added_ipv4) { print h1; added_ipv4=1 }
        next
    }
    /^# IPv6 local connections:/ {
        print
        if (!added_ipv6) { print h2; added_ipv6=1 }
        next
    }
    { print }
    END {
        if (!added_local) {
            print "\n# \"local\" is for Unix domain socket connections only"
            print l1
            print l2
        }
        if (!added_ipv4) {
            print "\n# IPv4 local connections:"
            print h1
        }
        if (!added_ipv6) {
            print "\n# IPv6 local connections:"
            print h2
        }
    }' "$pg_hba_conf" > "$pg_hba_conf.tmp" && mv "$pg_hba_conf.tmp" "$pg_hba_conf"
    chown postgres:postgres "$pg_hba_conf"
    chmod 600 "$pg_hba_conf"
    echo "pg_hba.conf has been updated successfully while preserving existing entries." >&3
}

remove_repo() {
  rm -f /etc/yum.repos.d/xf-app.repo
  rm -f /etc/yum.repos.d/fsr-connectors.repo
  rm -f /etc/yum.repos.d/fsr-pgdg.repo
}

change_file_ownership() {
  # reset all file ownership to FortiSOAR user in case new files have been created
  chown -R $agent_user:$agent_group $prefix
  restorecon -R $prefix
  chown -R $agent_user:$agent_group /var/log/xfagent/
  restorecon -R $prefix /var/log/xfagent
  chown -R $agent_user:$agent_group /opt/xf/configs/
  restorecon -R $prefix /opt/xf/configs
}

misc_agent_setup() {
    mkdir -p /opt/xf/configs/
    touch /opt/xf/configs/.config_changed
    touch /var/log/xfagent/xfagent.log
    chmod 777 /var/log/xfagent/xfagent.log
    touch /var/log/gunicorn.log
    chown $agent_user:$agent_group /var/log/gunicorn.log
    chmod 766 /var/log/gunicorn.log
}

add_venv_to_path() {
  touch ~/.profile
  prof=$(<~/.profile)
  echo $prof
  if [ -d /opt/integration/.env/bin ] && [[ $prof != *"/opt/integration/.env/bin"* ]]; then
    echo "[ -d /opt/integration/.env/bin ] && PATH=\"/opt/integration/.env/bin:\$PATH\"" >> ~/.profile
  fi
}

database_migrate() {
  echo "======================================================================================================"
  echo "STARTING DB MIGRATION"
  echo "======================================================================================================"
  cd "$prefix/" || { echo "Failed to change directory to $prefix"; return 1; }
  set -x  # Enable command tracing
  sudo -u "$agent_user" "$prefix/.env/bin/python3" /opt/integration/manage.py makemigrations || echo "Failed: makemigrations" >&2
  sudo -u "$agent_user" "$prefix/.env/bin/python3" /opt/integration/manage.py migrate || echo "Failed: migrate" >&2
  set +x  # Disable command tracing
}

create_tenant() {
  echo "======================================================================================================"
  echo "CREATING TENANT AND AGENT"
  echo "======================================================================================================"
  set -x  # Enable command tracing
  sudo -u "$agent_user" /opt/integration/.env/bin/python /opt/integration/manage.py create_xf_tenant --tenant_id "$tenant_id" --schema_name "$schema" --name "$tenant_id" --active True || echo "Failed: create_xf_tenant for tenant_id=$tenant_id, schema_name=$schema" >&2
  sudo -u "$agent_user" /opt/integration/.env/bin/python /opt/integration/manage.py create_self_agent_or_update_version --schema_name "$schema" || echo "Failed: create_self_agent_or_update_version for schema_name=$schema" >&2
  set +x  # Disable command tracing
}

enable_agent_service() {
  cp -r /opt/integration/package.d/gunicorn.service /etc/systemd/system/gunicorn.service
  systemctl daemon-reload
  systemctl enable gunicorn
}

restart_service() {
  # restart service
  echo "======================================================================================================"
  echo "RESTARTING SERVICES"
  echo "======================================================================================================"
  local pg_hba_conf="/var/lib/pgsql/${pg_version}/data/pg_hba.conf"
  if [ "$is_upgrade" = "false" ] && ! grep -iq "phoenix" "$pg_hba_conf"; then
    systemctl restart postgresql-$pg_version
  fi
  cd /opt/integration/
  source /etc/environment
  systemctl restart gunicorn.service
}

install_connectors() {
  sudo -u $agent_user $prefix/.env/bin/python3 /opt/integration/manage.py install_agent_connectors --schema_name $schema --filepath /opt/integration/connectors/agent_connectors.json
}

update_build_info() {
  rpm_version="$(rpm -q --qf \"%{VERSION}\" xf-integration-agent)"
  rpm_build_number="$(rpm -q --qf \"%{RELEASE}\" xf-integration-agent)"
  echo "{\"release-version\":$rpm_version,\"build-number\":$rpm_build_number}" > /opt/build-info.json
}

install_gcc() {
  rpm -q gcc >/dev/null
  if [ $? -ne 0 ]; then
    echo "======================================================================================================"
    echo "attempting to install gcc"
    echo "======================================================================================================"
    yum install -y gcc >/dev/null 2>&1

    if [ 0 -ne $? ]; then
      echo "======================================================================================================"
      echo "Failed to install GCC rpm. Some of the FortiSOAR connectors may have a dependency on gcc."
      echo "It is recommended to install gcc by enabling the CentOS yum repositories on this instance and running 'yum install gcc -y'"
      echo "======================================================================================================"
    else
      echo "successfully installed gcc"
    fi
  fi
}

logging_setup() {
  mkdir -p /var/log/xfagent
  f_log="/var/log/xfagent/xfagent-$agent_version-$time_stamp.log"
  touch $f_log
  f_file_descriptor_3="/dev/fd/3"
  exec 3>&1 1>>${f_log} 2>&1
}

get_ini_value() {
    local key=$1
    # Extract value using grep and sed. Adjust for different .ini file structures as needed.
    grep -oP "(?<=^$key=).*" "$INI_FILE"
}

main() {
  local pgpw
  local pguser
  local is_upgrade='false'
  check_if_upgrade
  if [ -z "$1" ]; then
    pgpw=""
    while [[ -z $pgpw ]]
    do
        get_password
    done
  elif [[ $1 == *.ini ]]; then
    INI_FILE="$1"
    pgpw=$(get_ini_value "pgpw")
    if [ -z $pgpw ]; then
      echo "No variable named 'pgpw' present in supplied .ini file. Exiting Agent Installer"
      exit 1
    elif [[ ! $pgpw =~ .*[a-z].* ]] || [[ ! $pgpw =~ .*[A-Z].* ]] || [[ ! $pgpw =~ .*[0-9].* ]] || [[ ! ${#pgpw} -ge 10 ]] || [[ ! ${#pgpw} -le 64 ]]; then
      echo "Passwords must contain one lowercase letter, one uppercase letter, and one numerical digit, and must be between 10 and 64 characters long. Please ensure your password meets these requirements and try again."
      exit 1
    fi
    pguser=$(get_ini_value "pguser")
  else
    echo "The supplied argument is not a .ini file. Exiting Agent Installer."
    exit 1
  fi
  if [ -z $pguser ]; then
    pguser="cyberpgsql"
  fi

  logging_setup
  set -x
  # Ensures the script fails if any command in the pipeline fails
  set -o pipefail
  {
    script_setup
    set_locale
    set_env_variables
    check_rmq_reachable
    #add_extra_hosts
    source /etc/environment
    add_xf_repo
    add_postgres_repo
    yum clean all
    install_agent_rpm
    #add_venv_to_path
    update_build_info
    misc_agent_setup
    populate_config_files
    populate_connectors_file
    disable_existing_postgres_repo
    #add_product_yum_server
    install_configure_postgres
    remove_repo
    change_file_ownership
    database_migrate
    create_tenant
    database_migrate
    enable_agent_service
    restart_service
    install_gcc
    install_connectors
  } | awk '{ print strftime("[%Y-%m-%d %H:%M:%S]"), $0; fflush(); }' | tee -a $f_file_descriptor_3
}

main $@
exit $?
