""" Copyright start
  Copyright (C) 2008 - 2025 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import logging

from django.conf import settings
from xfutils.constants import AUTHORIZATION_KEY, AUTH_TYPE_BEARER
from xfutils.utils import api_call, generate_interservice_jwt_token

from agents.constants import APPLIANCE_DELETE_URL, APPLIANCE_CREATE_URL, CONTENT_TYPE_JSON
from agents.exceptions import *
from constants.error_message import *

all_config = settings.ALL_CONFIG
logger = logging.getLogger(__name__)


def create_appliance_for_agent(agent_id, agent_name, tenant_id, tenant_uuid):
    try:
        payload = {
            'name': agent_name,
            'last_name': 'appliance',
            'type': 'appliance',
            'roles': ['Agent Appliance']
        }
        header = { 'Accept': CONTENT_TYPE_JSON, 'Content-Type': CONTENT_TYPE_JSON}
        create_user_url = APPLIANCE_CREATE_URL.format(all_config.get("application", "auth_host"))
        token = generate_interservice_jwt_token(all_config, tenantid=tenant_id, tenantuuid=tenant_uuid)
        header[AUTHORIZATION_KEY] = f"{AUTH_TYPE_BEARER} {token}"
        response = api_call(create_user_url, method='POST', body=payload, headers=header, verify=False)
        return response
    except Exception as e:
        message = xf_integration_36.format(agent_id)
        logger.exception(message)
        raise AgentCreationFailed(message)


def delete_agent_appliance(agent_id, appliance_uuids, tenant_id, tenant_uuid):
    try:
        payload = {'uuid' : appliance_uuids}
        header = {'Accept': CONTENT_TYPE_JSON, 'Content-Type': CONTENT_TYPE_JSON}
        delete_appliance_url = APPLIANCE_DELETE_URL.format(all_config.get("application", "auth_host"))

        token = generate_interservice_jwt_token(all_config, tenantid=tenant_id, tenantuuid=tenant_uuid)
        header[AUTHORIZATION_KEY] = f"{AUTH_TYPE_BEARER} {token}"

        response = api_call(delete_appliance_url, method='DELETE', body=payload, headers=header, verify=False)
        return response
    except Exception as e:
        message = xf_integration_59.format(agent_id)
        logger.exception(message)
        raise AgentCreationFailed(message)
